package api

import (
	"encoding/json"
	"fmt"
	"log"
	"net/http"

	"git.sp4ke.com/sp4ke/bit4sat/bus"
	"git.sp4ke.com/sp4ke/bit4sat/db"
	"git.sp4ke.com/sp4ke/bit4sat/ln"
	"git.sp4ke.com/sp4ke/bit4sat/storage"
	"git.sp4ke.com/sp4ke/bit4sat/utils"
	"github.com/gin-gonic/gin"
	"github.com/mediocregopher/radix/v3"
)

func invoiceCbHandler(c *gin.Context) {
	invoice := ln.Invoice{}

	//dec := json.Decoder(c.Request.Body)

	if err := c.ShouldBindJSON(&invoice); err != nil {
		log.Println(err)
		utils.JSONErr(c, http.StatusNotAcceptable,
			"callback: could not parse request from charge api")
		return
	}

	log.Printf("received invoice paid from ln charge\n%s", invoice)
	c.Status(http.StatusOK)

	// get upload id related to invoice
	var uploadId string
	invoiceUploadKey := fmt.Sprintf("invoice_%s_upload", invoice.Id)
	err := db.DB.Redis.Do(radix.FlatCmd(&uploadId, "GET", invoiceUploadKey))
	if err != nil {
		panic(err)
	}
	if uploadId == "" {
		log.Printf("cannot find uploadId for invoice %s", invoice)
		return
	}

	invoice.UploadId = uploadId
	//
	// Set upload status to UpWaitingStorage (paid)
	err = storage.SetUploadStatus(invoice.UploadId, storage.UpWaitingStorage)
	if err != nil {
		panic(err)
	}
	//
	// Update Upload Invoice
	err = storage.SetUploadInvoice(invoice.UploadId, &invoice)
	if err != nil {
		panic(err)
	}

	// publish invoice was updated to upload_id_paid channel
	msg := bus.Message{}
	msg.Type = bus.PaymentReceived
	msg.UploadId = invoice.UploadId
	msg.Data = gin.H{
		"invoice": invoice,
	}

	msgJson, err := json.Marshal(msg)
	if err != nil {
		panic(err)
	}

	log.Printf("Notifying upload paid %s", invoice.UploadId)
	key := fmt.Sprintf("%s_%s", bus.UploadUpdateChannelPrefix,
		invoice.UploadId)

	err = db.DB.Redis.Do(radix.FlatCmd(nil, "PUBLISH",
		key, msgJson))

	return
}
