package api

import (
	"git.sp4ke.com/sp4ke/bit4sat/config"
	"git.sp4ke.com/sp4ke/bit4sat/storage"
	"git.sp4ke.com/sp4ke/bit4sat/ws"
	"github.com/gin-contrib/cors"
	"github.com/gin-contrib/sessions"
	"github.com/gin-contrib/sessions/redis"
	"github.com/gin-gonic/gin"
)

var (
	UploadCtrl = storage.UploadCtrl{}
)

type API struct {
	router *gin.Engine
}

func (api *API) Run() {

	uploadRoute := api.router.Group("/api/upload")
	{
		uploadRoute.POST("", UploadCtrl.New)
		uploadRoute.PUT(":id", UploadCtrl.Upload)
	}

	// Websocket server
	api.router.GET("/ws", ws.Serve)

	// LN charge callback
	api.router.POST("/"+config.ChargeCallbackEndpoint, invoiceCbHandler)

	api.router.Run(config.ApiListen)
}

func NewAPI() *API {
	router := gin.Default()
	router.Use(cors.Default())

	// Setup Session
	sessionStore, err := redis.NewStore(10, "tcp", "redis:6379",
		"", []byte(config.SessionSecret))
	if err != nil {
		panic(err)
	}

	router.Use(sessions.Sessions("websocket-session", sessionStore))

	//
	//
	//router.Use(secure.New(secure.Config{
	//ContentSecurityPolicy: "default-src 'self'; script-src *; worker-src *",
	////IsDevelopment: true,
	//}))

	return &API{
		router: router,
	}
}
